commit 2f2d0088eb93 ("usbip: prevent vhci_hcd driver from leaking a
socket pointer address")

When a client has a USB device attached over IP, the vhci_hcd driver is
locally leaking a socket pointer address via the

/sys/devices/platform/vhci_hcd/status file (world-readable) and in debug
output when "usbip --debug port" is run.

Fix it to not leak. The socket pointer address is not used at the moment
and it was made visible as a convenient way to find IP address from
socket pointer address by looking up /proc/net/{tcp,tcp6}.

As this opens a security hole, the fix replaces socket pointer address
with sockfd.

Reported-by: Secunia Research <v...@secunia.com>
Signed-off-by: Shuah Khan <shua...@osg.samsung.com>
---
 drivers/usb/usbip/usbip_common.h     |  1 +
 drivers/usb/usbip/vhci_sysfs.c       | 25 +++++++++++++++----------
 tools/usb/usbip/libsrc/vhci_driver.c |  8 ++++----
 3 files changed, 20 insertions(+), 14 deletions(-)

diff --git a/drivers/usb/usbip/usbip_common.h b/drivers/usb/usbip/usbip_common.h
index 9f490375ac92..f0b955f8504e 100644
--- a/drivers/usb/usbip/usbip_common.h
+++ b/drivers/usb/usbip/usbip_common.h
@@ -271,6 +271,7 @@ struct usbip_device {
        /* lock for status */
        spinlock_t lock;
 
+       int sockfd;
        struct socket *tcp_socket;
 
        struct task_struct *tcp_rx;
diff --git a/tools/usb/usbip/libsrc/vhci_driver.c 
b/tools/usb/usbip/libsrc/vhci_driver.c
index ad9204773533..1274f326242c 100644
--- a/tools/usb/usbip/libsrc/vhci_driver.c
+++ b/tools/usb/usbip/libsrc/vhci_driver.c
@@ -55,12 +55,12 @@ static int parse_status(const char *value)
 
        while (*c != '\0') {
                int port, status, speed, devid;
-               unsigned long socket;
+               int sockfd;
                char lbusid[SYSFS_BUS_ID_SIZE];
 
-               ret = sscanf(c, "%d %d %d %x %lx %31s\n",
+               ret = sscanf(c, "%d %d %d %x %u %31s\n",
                                &port, &status, &speed,
-                               &devid, &socket, lbusid);
+                               &devid, &sockfd, lbusid);
 
                if (ret < 5) {
                        dbg("sscanf failed: %d", ret);
@@ -69,7 +69,7 @@ static int parse_status(const char *value)
 
                dbg("port %d status %d speed %d devid %x",
                                port, status, speed, devid);
-               dbg("socket %lx lbusid %s", socket, lbusid);
+               dbg("sockfd %u lbusid %s", sockfd, lbusid);
 
 
                /* if a device is connected, look at it */
diff --git a/drivers/usb/usbip/vhci_sysfs.c b/drivers/usb/usbip/vhci_sysfs.c
index b96e5b189269..c287ccc78fde 100644
--- a/drivers/usb/usbip/vhci_sysfs.c
+++ b/drivers/usb/usbip/vhci_sysfs.c
@@ -49,13 +49,17 @@ static ssize_t status_show_vhci(int pdev_nr, char *out)
 
        /*
         * output example:
         * port sta spd dev      socket           local_busid
         * 0000 004 000 00000000         c5a7bb80 1-2.3
         * 0001 004 000 00000000         d8cee980 2-3.4
         *
-        * IP address can be retrieved from a socket pointer address by looking
-        * up /proc/net/{tcp,tcp6}. Also, a userland program may remember a
-        * port number and its peer IP address.
+        * Output includes socket fd instead of socket pointer address to
+        * avoid leaking kernel memory address in:
+        *      /sys/devices/platform/vhci_hcd.0/status and in debug output.
+        * The socket pointer address is not used at the moment and it was
+        * made visible as a convenient way to find IP address from socket
+        * pointer address by looking up /proc/net/{tcp,tcp6}. As this opens
+        * a security hole, the change is made to use sockfd instead.
         */
        for (i = 0; i < VHCI_HC_PORTS; i++) {
                struct vhci_device *vdev = &vhci->vdev[i];
@@ -68,13 +72,13 @@ static ssize_t status_show_vhci(int pdev_nr, char *out)
                if (vdev->ud.status == VDEV_ST_USED) {
                        out += sprintf(out, "%03u %08x ",
                                            vdev->speed, vdev->devid);
-                       out += sprintf(out, "%16p %s",
-                                           vdev->ud.tcp_socket,
+                       out += sprintf(out, "%06u %s",
+                                           vdev->ud.sockfd,
                                            dev_name(&vdev->udev->dev));
 
                } else {
                        out += sprintf(out, "000 00000000 ");
-                       out += sprintf(out, "0000000000000000 0-0");
+                       out += sprintf(out, "000000 0-0");
                }
 
                out += sprintf(out, "\n");
@@ -125,7 +129,7 @@ static ssize_t status_show(struct device *dev,
        int pdev_nr;
 
        out += sprintf(out,
-                      "port sta spd dev      socket           local_busid\n");
+                      "port sta spd dev      sockfd local_busid\n");
 
        pdev_nr = status_name_to_id(attr->attr.name);
        if (pdev_nr < 0)
@@ -324,6 +328,7 @@ static ssize_t store_attach(struct device *dev, struct 
device_attribute *attr,
 
        vdev->devid         = devid;
        vdev->speed         = speed;
+       vdev->ud.sockfd     = sockfd;
        vdev->ud.tcp_socket = socket;
        vdev->ud.status     = VDEV_ST_NOTASSIGNED;
 
@@ -49,13 +49,17 @@ static ssize_t status_show_vhci(int pdev_nr, char *out)
        /*
         * output example:
-        * port sta spd dev      socket           local_busid
-        * 0000 004 000 00000000         c5a7bb80 1-2.3
-        * 0001 004 000 00000000         d8cee980 2-3.4
+        * port sta spd dev      sockfd local_busid
+        * 0000 004 000 00000000 000003 1-2.3
+        * 0001 004 000 00000000 000004 2-3.4
         *
-- 
2.14.1

Reply via email to